/*
 * THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
 * ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 * Copyright (C) NEC Corporation 2002.
 *
 *
 * Flash memory access on VR4181A based devices.
 *	Intel Strata Flash (28F128-150)
 *	NEC Ostrich:	32MiB FROM (128Bib x2) 16bit width.
 *	NEC Alba:	32MiB/64MiB FROM 16/32 bit width.
 */

#include <linux/config.h>
#ifndef CONFIG_NEC_SMVR4181A
#error This is for NEC SMVR418A board only
#else
#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>

#include <asm/vr4181a/vr4181a.h>

#define WINDOW_ADDR	0xbe000000
#define WINDOW_SIZE	0x02000000

static void smvr4181a_set_vpp(struct map_info *map, int vpp)
{
	vr4181a_set_gpio_mode(42, GPIO_OUTPUT, GPIO_INPUT_PROHIBIT);

	if(vpp)
		vr4181a_set_gpio_data(42, 1);
	else
		vr4181a_set_gpio_data(42, 0);
}

static __u8 smvr4181a_read8(struct map_info *map, unsigned long ofs)
{
	return *(__u8 *)(map->map_priv_1 + ofs);
}

static __u16 smvr4181a_read16(struct map_info *map, unsigned long ofs)
{
	return *(__u16 *)(map->map_priv_1 + ofs);
}

static __u32 smvr4181a_read32(struct map_info *map, unsigned long ofs)
{
	return *(__u32 *)(map->map_priv_1 + ofs);
}

static void smvr4181a_copy_from(struct map_info *map, void *to, unsigned long from, ssize_t len)
{
	memcpy(to, (void *)(map->map_priv_1 + from), len);
}

static void smvr4181a_write8(struct map_info *map, __u8 d, unsigned long adr)
{
	*(__u8 *)(map->map_priv_1 + adr) = d;
}

static void smvr4181a_write16(struct map_info *map, __u16 d, unsigned long adr)
{
	*(__u16 *)(map->map_priv_1 + adr) = d;
}

static void smvr4181a_write32(struct map_info *map, __u32 d, unsigned long adr)
{
	*(__u32 *)(map->map_priv_1 + adr) = d;
}

static void smvr4181a_copy_to(struct map_info *map, unsigned long to, const void *from, ssize_t len)
{
	memcpy((void *)(map->map_priv_1 + to), from, len);
}

static struct map_info smvr4181a_map = {
	name:		"SMVR4181A on-board FlashROM",
	buswidth:	2,
	size:		WINDOW_SIZE,
	read8:		smvr4181a_read8,
	read16:		smvr4181a_read16,
	read32:		smvr4181a_read32,
	copy_from:	smvr4181a_copy_from,
	write8:		smvr4181a_write8,
	write16:	smvr4181a_write16,
	write32:	smvr4181a_write32,
	copy_to:	smvr4181a_copy_to,
	set_vpp:	smvr4181a_set_vpp,
	map_priv_1:	WINDOW_ADDR
};

static unsigned long smvr4181a_max_flash_size = 0x02000000;

static struct mtd_partition smvr4181a_partitions[] = {
	{
		name:	"FlashROM",
		offset:	0x00000000,
		size:	0x01c00000,	/* 28M Byte */
	},
#if 0
	{
		name:	"monitor",
		offset:	0x01c00000,
		size:	0x00100000,	/* 1M Byte */
		mask_flags:	MTD_CAP_ROM,
	},
	{
		name:	"kernel",
		offset:	0x01d00000,
		size:	0x00300000,	/* 3M Byte */
	},
#endif
};

#define NB_OF(x)	(sizeof(x)/sizeof(x[0]))

static struct mtd_info *mymtd;

static int __init smvr4181a_mtd_init(void)
{
	struct mtd_partition *parts;
	int nb_parts = 0;

	parts = smvr4181a_partitions;
	nb_parts = NB_OF(smvr4181a_partitions);
	smvr4181a_map.size = smvr4181a_max_flash_size;

	mymtd = do_map_probe("cfi_probe", &smvr4181a_map); 
	if (!mymtd)
		return -ENXIO;
	mymtd->module = THIS_MODULE;
	add_mtd_partitions(mymtd, parts, nb_parts);
		
	printk(KERN_INFO "SMVR181A flash access initialized\n");
	return 0;
}

static void __exit smvr4181a_mtd_cleanup(void)
{
	if (mymtd) {
		del_mtd_partitions(mymtd);
		map_destroy(mymtd);
	}
}

module_init(smvr4181a_mtd_init);
module_exit(smvr4181a_mtd_cleanup);

#endif /* CONFIG_NEC_SMVR4181A */
